package cc.fascinated.bat.features.leveling;

import cc.fascinated.bat.common.ChannelUtils;
import cc.fascinated.bat.common.Serializable;
import com.google.gson.Gson;
import lombok.Setter;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import org.bson.Document;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
@Setter
public class LevelingProfile extends Serializable {
    /**
     * The levels of each user.
     */
    private final Map<String, UserLevel> userLevels = new HashMap<>();

    /**
     * The id of the level up notification channel.
     */
    private String notificationChannelId;

    /**
     * Gets the user level of a user.
     *
     * @param userId The user ID.
     * @return The user level.
     */
    public UserLevel getUserLevel(String userId) {
        return userLevels.computeIfAbsent(userId, s -> new UserLevel(1, 0, -1));
    }

    /**
     * Gets the notification channel.
     *
     * @return The notification channel.
     */
    public TextChannel getNotificationChannel() {
        return ChannelUtils.getTextChannel(notificationChannelId);
    }

    @Override
    public void load(Document document, Gson gson) {
        for (String userId : document.keySet()) {
            Document userDocument = document.get(userId, Document.class);
            userLevels.put(userId, new UserLevel(
                    userDocument.getInteger("level"),
                    userDocument.getDouble("currentXp"),
                    (long) userDocument.getOrDefault("lastXpTimestamp", (long) -1)
            ));
        }
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        for (Map.Entry<String, UserLevel> entry : userLevels.entrySet()) {
            document.put(entry.getKey(), new Document()
                    .append("level", entry.getValue().getLevel())
                    .append("currentXp", entry.getValue().getCurrentXp())
                    .append("lastXpTimestamp", entry.getValue().getLastXpTimestamp())
            );
        }
        return document;
    }

    @Override
    public void reset() {
        userLevels.clear();
    }
}
