package cc.fascinated.bat.features.leveling;

import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor
@Getter
public class UserLevel {
    /**
     * The current level of the user.
     */
    private int level;

    /**
     * The current XP of the user.
     */
    private double currentXp;

    /**
     * The timestamp of the last message sent by the user.
     */
    private long lastXpTimestamp;

    /**
     * Adds XP to the user.
     */
    public void addXp(double xp) {
        this.currentXp += xp;
    }

    /**
     * Checks if the user can level up.
     *
     * @return If the user can level up.
     */
    public boolean canLevelUp() {
        // Handle XP cooldown
        long cooldown = System.currentTimeMillis() - this.lastXpTimestamp;
        if (cooldown < 30_000) {
            return false;
        }
        this.lastXpTimestamp = System.currentTimeMillis();
        return this.currentXp >= LevelingFeature.getNeededXP(this.level + 1, this.currentXp);
    }

    /**
     * Levels up the user.
     */
    public void levelup() {
        do {
            if (this.currentXp < LevelingFeature.getNeededXP(this.level + 1, this.currentXp)) {
                break;
            }
            this.level++;
        } while (this.currentXp >= LevelingFeature.getNeededXP(this.level + 1, this.currentXp));
    }

    /**
     * Resets the user's level and XP.
     */
    public void reset() {
        this.level = 1;
        this.currentXp = 0;
    }
}
