package cc.fascinated.bat.features.namehistory;

import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.namehistory.profile.user.NameHistoryProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.FeatureService;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.events.guild.member.update.GuildMemberUpdateNicknameEvent;
import net.dv8tion.jda.api.events.user.update.UserUpdateGlobalNameEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class NameHistoryListener implements EventListener {
    private final GuildService guildService;
    private final FeatureService featureService;

    @Autowired
    public NameHistoryListener(@NonNull GuildService guildService, @NonNull FeatureService featureService) {
        this.guildService = guildService;
        this.featureService = featureService;
    }

    @Override
    public void onUserUpdateGlobalName(@NonNull BatUser user, String oldName, String newName, @NonNull UserUpdateGlobalNameEvent event) {
        NameHistoryProfile profile = user.getNameHistoryProfile();
        profile.addName(newName);
    }

    @Override
    public void onGuildMemberUpdateNickname(@NonNull BatGuild guild, @NonNull BatUser user, String oldName, String newName,
                                            @NonNull GuildMemberUpdateNicknameEvent event) {
        NameHistoryFeature nameHistoryFeature = featureService.getFeature(NameHistoryFeature.class);
        if (!guild.getFeatureProfile().isFeatureEnabled(nameHistoryFeature)) { // Check if the feature is enabled
            return;
        }

        cc.fascinated.bat.features.namehistory.profile.guild.NameHistoryProfile profile = guild.getNameHistoryProfile();
        profile.addName(user, newName);
    }
}
