package cc.fascinated.bat.features.reminder.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.TimeUtils;
import cc.fascinated.bat.features.reminder.Reminder;
import cc.fascinated.bat.features.reminder.ReminderFeature;
import cc.fascinated.bat.features.reminder.ReminderProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * @author Fascinated (fascinated7)
 */
@Component("reminder:set.sub")
@CommandInfo(name = "set", description = "Set a reminder.")
public class SetSubCommand extends BatCommand {
    public SetSubCommand() {
        super.addOptions(
                new OptionData(OptionType.STRING, "reminder", "The reminder to set.", true),
                new OptionData(OptionType.STRING, "time", "After how long should the reminder be sent. (eg: 5m)", true)
        );
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        ReminderProfile profile = guild.getReminderProfile();
        if (profile.getReminderCount(user.getDiscordUser()) >= ReminderFeature.MAX_REMINDERS) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("You have reached the maximum amount of reminders.")
                    .build()).queue();
            return;
        }

        OptionMapping reminderOption = event.getOption("reminder");
        assert reminderOption != null;
        OptionMapping timeOption = event.getOption("time");
        assert timeOption != null;

        String reminderText = reminderOption.getAsString();
        long time = TimeUtils.fromString(timeOption.getAsString());
        if (time == 0) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Invalid time format.")
                    .build()).queue();
            return;
        }
        if (time < TimeUnit.MINUTES.toMillis(1)) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The time must be at least 1 minute.")
                    .build()).queue();
            return;
        }
        if (time > ReminderFeature.MAX_REMINDER_LENGTH) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The time must be at most %s.".formatted(TimeUtils.format(ReminderFeature.MAX_REMINDER_LENGTH,
                            TimeUtils.BatTimeFormat.FIT, true)))
                    .build()).queue();
            return;
        }
        Reminder reminder = profile.addReminder(user.getDiscordUser(), event.getChannel().asTextChannel(), reminderText, new Date(System.currentTimeMillis() + time));

        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Reminder for `%s` set, you will be reminded <t:%s:R>".formatted(reminderText,
                        reminder.getEndDate().toInstant().getEpochSecond()))
                .build()).queue();
    }
}
