package cc.fascinated.bat.features.spotify.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.spotify.SpotifyFeature;
import cc.fascinated.bat.features.spotify.profile.SpotifyProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.SpotifyService;
import lombok.NonNull;
import lombok.SneakyThrows;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "unlink", description = "Unlink your Spotify account")
public class UnlinkSubCommand extends BatCommand implements EventListener {
    private final SpotifyService spotifyService;

    @Autowired
    public UnlinkSubCommand(@NonNull SpotifyService spotifyService) {
        this.spotifyService = spotifyService;
    }

    @Override @SneakyThrows
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        SpotifyProfile profile = user.getProfile(SpotifyProfile.class);
        if (!profile.hasLinkedAccount() || !spotifyService.hasTrackPlaying(user)) {
            event.replyEmbeds(SpotifyFeature.checkSpotify(spotifyService, user).build()).queue();
            return;
        }

        profile.reset();
        event.replyEmbeds(EmbedUtils.successEmbed()
                        .setDescription("%s Successfully unlinked your Spotify account.".formatted(Emojis.CHECK_MARK_EMOJI))
                        .build())
                .setEphemeral(true)
                .queue();
    }
}
