package cc.fascinated.bat.features.statschannel;

import cc.fascinated.bat.features.Feature;
import cc.fascinated.bat.features.FeatureProfile;
import cc.fascinated.bat.features.statschannel.command.StatsChannelCommand;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.service.CommandService;
import cc.fascinated.bat.service.DiscordService;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Guild;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "Stats Channel Feature")
public class StatsChannelFeature extends Feature {
    private final GuildService guildService;

    @Autowired
    public StatsChannelFeature(@NonNull ApplicationContext context, @NonNull CommandService commandService, @NonNull GuildService guildService) {
        super("Stats Channels", FeatureProfile.FeatureState.DISABLED, true);
        this.guildService = guildService;

        super.registerCommand(commandService, context.getBean(StatsChannelCommand.class));
    }

    /**
     * Scheduled task to update the stats channel
     * This will run every hour
     */
    @Scheduled(cron = "0 0 * * * *")
    public void updateStatsChannel() {
        for (Guild guild : DiscordService.JDA.getGuilds()) {
            BatGuild batGuild = guildService.getGuild(guild.getId());
            assert batGuild != null;
            if (batGuild.getFeatureProfile().isFeatureDisabled(this)) {
                continue;
            }
            StatsChannelProfile profile = batGuild.getStatsChannelProfile();
            profile.refreshChannels(batGuild);
        }
    }

    /**
     * Get the placeholders that the user can use
     *
     * @param replacements What to replace the placeholders using
     * @return The placeholders
     */
    public static String getPlaceholders(Object... replacements) {
        StringBuilder builder = new StringBuilder();
        builder.append("**Available Placeholders:**\n");
        for (StatsPlaceholders placeholder : StatsPlaceholders.values()) {
            String placeholderString = placeholder.getPlaceholder();
            builder.append("`").append(placeholderString).append("` - ")
                    .append(StatsPlaceholders.replaceAllPlaceholders(placeholderString, replacements)).append("\n");
        }
        return builder.toString();
    }
}
