package cc.fascinated.bat.features.statschannel;

import cc.fascinated.bat.common.Serializable;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.service.DiscordService;
import com.google.gson.Gson;
import net.dv8tion.jda.api.entities.channel.concrete.VoiceChannel;
import org.bson.Document;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
public class StatsChannelProfile extends Serializable {
    /**
     * The channels to show stats on.
     */
    private final Map<String, StatsChannel> channels = new HashMap<>();

    /**
     * Gets the channels to show stats on.
     *
     * @return the channels
     */
    public Map<VoiceChannel, StatsChannel> getChannels() {
        Map<VoiceChannel, StatsChannel> textChannels = new HashMap<>();
        for (Map.Entry<String, StatsChannel> entry : channels.entrySet()) {
            VoiceChannel textChannel = DiscordService.JDA.getVoiceChannelById(entry.getKey());
            if (textChannel != null) {
                textChannels.put(textChannel, entry.getValue());
            }
        }
        return textChannels;
    }

    /**
     * Gets a channel
     *
     * @param id the id of the channel
     * @return the channel
     */
    public VoiceChannel getChannel(String id) {
        if (!channels.containsKey(id)) {
            return null;
        }
        return DiscordService.JDA.getVoiceChannelById(id);
    }

    /**
     * Adds a new stats channel
     *
     * @param channel the channel
     * @param text the text to display
     */
    public void addChannel(VoiceChannel channel, String text) {
        channels.put(channel.getId(), new StatsChannel(text, ""));
    }

    /**
     * Removes a channel
     *
     * @param channel the channel
     */
    public void removeChannel(VoiceChannel channel) {
        channels.remove(channel.getId());
    }

    /**
     * Updates all the stats channels for the guild
     */
    public void refreshChannels(BatGuild guild) {
        for (Map.Entry<String, StatsChannel> entry : this.channels.entrySet()) {
            VoiceChannel channel = DiscordService.JDA.getVoiceChannelById(entry.getKey());
            if (channel == null) {
                continue;
            }
            StatsChannel statsChannel = entry.getValue();
            String display = StatsPlaceholders.replaceAllPlaceholders(statsChannel.getDisplay(), guild);
            if (statsChannel.getLastValue().equals(display)) { // Don't update if the value is the same
                continue;
            }
            channel.getManager().setName(display).queue();
        }
    }

    @Override
    public void load(Document document, Gson gson) {
        Document channelsDocument = (Document) document.getOrDefault("channels", new Document());
        for (Map.Entry<String, Object> entry : channelsDocument.entrySet()) {
            Document channelDocument = (Document) entry.getValue();
            channels.put(entry.getKey(), new StatsChannel(
                    channelDocument.getString("display"),
                    channelDocument.getString("lastValue")
            ));
        }
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        Document channelsDocument = new Document();
        for (Map.Entry<String, StatsChannel> entry : channels.entrySet()) {
            StatsChannel statsChannel = entry.getValue();
            Document channelDocument = new Document();
            channelDocument.put("display", statsChannel.getDisplay());
            channelDocument.put("lastValue", statsChannel.getLastValue());
            channelsDocument.put(entry.getKey(), channelDocument);
        }
        document.put("channels", channelsDocument);
        return document;
    }

    @Override
    public void reset() {
        channels.clear();
    }
}
