package cc.fascinated.bat.features.statschannel.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.statschannel.StatsChannelProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

import java.util.Collections;

/**
 * @author Fascinated (fascinated7)
 */
@Component("stats-channel.add:sub")
@CommandInfo(
        name = "add",
        description = "Add a stats channel"
)
public class AddSubCommand extends BatCommand {
    public AddSubCommand() {
        super.addOptions(new OptionData(OptionType.STRING, "display", "The text to display in the stats channel", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping displayOption = event.getOption("display");
        assert displayOption != null;

        String display = displayOption.getAsString();
        if (display.length() > 24) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("%s The channel name can only be a maximum of 32 characters".formatted(Emojis.CROSS_MARK_EMOJI))
                    .build()).queue();
            return;
        }

        StatsChannelProfile profile = guild.getStatsChannelProfile();
        if (profile.getChannels().size() >= 5) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("%s You can only have a maximum of 5 stats channels".formatted(Emojis.CROSS_MARK_EMOJI))
                    .build()).queue();
            return;
        }

        guild.getDiscordGuild().createVoiceChannel("stats channel").queue(voiceChannel -> {
            // Disallow everyone from connecting
            voiceChannel.getManager().putPermissionOverride(
                    guild.getDiscordGuild().getPublicRole(),
                    null,
                    Collections.singleton(Permission.VOICE_CONNECT)
            ).queue();

            profile.addChannel(voiceChannel, display);
            profile.refreshChannels(guild); // For the stat channels to be updated

            event.replyEmbeds(EmbedUtils.successEmbed()
                    .setDescription(new EmbedDescriptionBuilder("%s The stats chanel %s has been created".formatted(
                            Emojis.CHECK_MARK_EMOJI,
                            voiceChannel.getAsMention()
                    ))
                            .appendLine("Display: `%s`".formatted(display), true)
                            .build())
                    .build()).queue();
        });
    }
}
