package cc.fascinated.bat.features.statschannel.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.statschannel.StatsChannelProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.ChannelType;
import net.dv8tion.jda.api.entities.channel.concrete.VoiceChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.entities.channel.unions.GuildChannelUnion;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("stats-channel.remove:sub")
@CommandInfo(
        name = "remove",
        description = "Remove a stats channel"
)
public class RemoveSubCommand extends BatCommand {
    public RemoveSubCommand() {
        super.addOptions(new OptionData(OptionType.CHANNEL, "channel", "The stats channel to remove", true));
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping channelOption = event.getOption("channel");
        assert channelOption != null;

        GuildChannelUnion channelUnion = channelOption.getAsChannel();
        if (channelUnion.getType() != ChannelType.VOICE) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("%s The channel must be a voice channel".formatted(Emojis.CROSS_MARK_EMOJI))
                    .build()).queue();
            return;
        }

        StatsChannelProfile profile = guild.getStatsChannelProfile();
        VoiceChannel statsChannel = profile.getChannel(channelUnion.getId());
        if (statsChannel == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("%s The channel is not a stats channel".formatted(Emojis.CROSS_MARK_EMOJI))
                    .build()).queue();
            return;
        }

        statsChannel.delete().queue(then -> {
            profile.removeChannel(statsChannel);
            event.replyEmbeds(EmbedUtils.successEmbed()
                    .setDescription("%s The stats channel has been removed".formatted(Emojis.CHECK_MARK_EMOJI))
                    .build()).queue();
        });
    }
}
